@@ -19,7 +19,7 @@
 
 #include <stddef.h>
 
-#include <asoundlib.h>
+#include <alsa/asoundlib.h>
 
 #include <spa/node/node.h>
 #include <spa/param/audio/format.h>
