@@ -23,7 +23,7 @@
 
 extern const struct spa_handle_factory spa_videotestsrc_factory;
 
-int
+SPA_EXPORT int
 spa_handle_factory_enum(const struct spa_handle_factory **factory, uint32_t *index)
 {
 	spa_return_val_if_fail(factory != NULL, -EINVAL);
