@@ -95,6 +95,10 @@ struct impl {
 #define RTKIT_SERVICE_NAME "org.freedesktop.RealtimeKit1"
 #define RTKIT_OBJECT_PATH "/org/freedesktop/RealtimeKit1"
 
+#ifndef RLIMIT_RTTIME
+#define RLIMIT_RTTIME 15
+#endif
+
 /** \cond */
 struct pw_rtkit_bus {
 	DBusConnection *bus;
@@ -106,6 +110,9 @@ struct pw_rtkit_bus *pw_rtkit_bus_get_system(void)
 	struct pw_rtkit_bus *bus;
 	DBusError error;
 
+	if (getenv("DISABLE_RTKIT"))
+		return NULL;
+
 	dbus_error_init(&error);
 
 	bus = calloc(1, sizeof(struct pw_rtkit_bus));
@@ -404,6 +411,8 @@ static void idle_func(struct spa_source *source)
 	long long rttime;
 	uint64_t count;
 
+	read(impl->source.fd, &count, sizeof(uint64_t));
+
 	rtprio = 20;
 	rttime = 20000;
 
@@ -414,7 +423,10 @@ static void idle_func(struct spa_source *source)
 		pw_log_debug("SCHED_OTHER|SCHED_RESET_ON_FORK worked.");
 		return;
 	}
+
 	system_bus = pw_rtkit_bus_get_system();
+	if (system_bus == NULL)
+		return;
 
 	rl.rlim_cur = rl.rlim_max = rttime;
 	if ((r = setrlimit(RLIMIT_RTTIME, &rl)) < 0)
@@ -437,8 +449,6 @@ static void idle_func(struct spa_source *source)
 		pw_log_debug("thread made realtime");
 	}
 	pw_rtkit_bus_free(system_bus);
-
-	read(impl->source.fd, &count, sizeof(uint64_t));
 }
 
 static int module_init(struct pw_module *module, struct pw_properties *properties)
@@ -478,6 +488,7 @@ static int module_init(struct pw_module *module, struct pw_properties *propertie
 	return 0;
 }
 
+SPA_EXPORT
 int pipewire__module_init(struct pw_module *module, const char *args)
 {
 	return module_init(module, NULL);
